FUNCTION = 'system.function'
STORAGE = 'system.storage'
COLLECTOR = 'system.collector'
WORKFLOW = 'system.workflow'
DESTINATION = 'system.destination'
LOGGER = 'system.logger'
PROPERTIES = 'system.event.properties'
ATTACHMENTS = 'system.event.attachments'
ENTITY_MERGING = 'system.entity.merging'